package eu.jonahbauer.json.tokenizer.reader;

import org.jetbrains.annotations.NotNull;

import java.io.IOException;
import java.io.Reader;
import java.util.Objects;

public class PushbackReaderImpl implements PushbackReader {
    private final @NotNull Reader reader;

    private int current = Integer.MIN_VALUE;
    private int lineNumber = 1;
    private int columnNumber = 0; // column number will be incremented when a character is read

    private boolean pushback;

    public PushbackReaderImpl(@NotNull Reader reader) {
        this.reader = Objects.requireNonNull(reader, "reader");
    }

    @Override
    public int getLineNumber() {
        if (current == Integer.MIN_VALUE) {
            throw new IllegalStateException("No character has been read so far.");
        }
        return lineNumber;
    }

    @Override
    public int getColumnNumber() {
        if (current == Integer.MIN_VALUE) {
            throw new IllegalStateException("No character has been read so far.");
        }
        return columnNumber;
    }

    @Override
    public void pushback() {
        if (current == Integer.MIN_VALUE) {
            throw new IllegalStateException("No character has been read so far.");
        } else if (pushback) {
            throw new IllegalStateException("Cannot push back more than one character at a time.");
        }
        pushback = true;
    }

    @Override
    public int read() throws IOException {
        if (pushback) {
            pushback = false;
            return current;
        }

        int result = reader.read();

        if (current == '\n' && result != '\n' || current == '\r' && result != '\n' && result != '\r') {
            lineNumber++;
            columnNumber = 1;
        } else {
            columnNumber++;
        }

        current = result;
        return result;
    }

    @Override
    public void close() throws IOException {
        reader.close();
    }
}
